/*
 * CPU detection code.
 *
 * Copyright (c) 2019 Loongson Technology Corporation Limited
 *                    Shiyou Yin <yinshiyou-hf@loongson.cn>
 *
 * This file is part of FFmpeg.
 *
 * FFmpeg is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * FFmpeg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <sys/wait.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/resource.h>
#include "libavutil/mips/cpu.h"
#include "libavutil/cpu.h"
#include "libavutil/cpu_internal.h"

#define MMI_MASK    0x00000010

static int get_cpu_flags_from_cpuinfo(void)
{
    int flags = 0;
    int model_name_flag = 0;
    char buf[1024];

# ifdef __linux__
    FILE* fp = fopen("/proc/cpuinfo", "r");
    if (!fp)
        return flags;

    memset(buf, 0, sizeof(buf));
    while (fgets(buf, sizeof(buf), fp)) {
        if(model_name_flag && !strncmp(buf, "ASEs implemented", strlen("ASEs implemented"))) {
            if (strstr(buf, "loongson-mmi") || strstr(buf, "loongson-ext")) {
                flags |= AV_CPU_FLAG_MMI;
            }
            if (strstr(buf, "msa")) {
                flags |= AV_CPU_FLAG_MSA;
            }
            break;
        } else if (!strncmp(buf, "model name", strlen("model name"))) {
            /* In case of some kernel havn't add loongson extention in ASEs implemented,
             * analysis model name is still needed.
             */
            if (strstr(buf, "3A4000")) {
                flags |= AV_CPU_FLAG_MSA | AV_CPU_FLAG_MMI;
            } else if (strstr(buf, "Loongson-3")) {
                flags |= AV_CPU_FLAG_MMI;
            } else if (strstr(buf, "Loongson-2K")  || strstr(buf, "2K1000")) {
                flags |= AV_CPU_FLAG_MSA | AV_CPU_FLAG_MMI;
            }
            model_name_flag = 1;
        }
    }

    fclose(fp);
# endif
    return flags;
}

/* Function to test if multimedia instructions are supported...  */
int ff_get_cpu_flags_mips(void)
{
    int flags = 0;

    /* TODO: pending to add get_cpu_flags_from_cpucfg. */
    flags = get_cpu_flags_from_cpuinfo();
    return flags;
}
